// Copyright 2020-2023, Collabora, Ltd.
// SPDX-License-Identifier: BSL-1.0
/*!
 * @file
 * @brief  Generated IPC client code.
 * @author Jakob Bornecrantz <jakob@collabora.com>
 * @ingroup ipc_client
 */

#pragma once

#include "shared/ipc_protocol.h"
#include "ipc_protocol_generated.h"
#include "client/ipc_client.h"


#ifdef __cplusplus
extern "C" {
#endif


xrt_result_t
ipc_call_instance_get_shm_fd(struct ipc_connection *ipc_c,
                             xrt_shmem_handle_t *handles,
                             uint32_t handle_count);

xrt_result_t
ipc_call_instance_describe_client(struct ipc_connection *ipc_c,
                                  const struct ipc_client_description *desc);

xrt_result_t
ipc_call_system_get_properties(struct ipc_connection *ipc_c,
                               struct xrt_system_properties *out_properties);

xrt_result_t
ipc_call_system_get_client_info(struct ipc_connection *ipc_c,
                                uint32_t id,
                                struct ipc_app_state *out_ias);

xrt_result_t
ipc_call_system_get_clients(struct ipc_connection *ipc_c,
                            struct ipc_client_list *out_clients);

xrt_result_t
ipc_call_system_set_primary_client(struct ipc_connection *ipc_c,
                                   uint32_t id);

xrt_result_t
ipc_call_system_set_focused_client(struct ipc_connection *ipc_c,
                                   uint32_t id);

xrt_result_t
ipc_call_system_toggle_io_client(struct ipc_connection *ipc_c,
                                 uint32_t id);

xrt_result_t
ipc_call_system_toggle_io_device(struct ipc_connection *ipc_c,
                                 uint32_t id);

xrt_result_t
ipc_call_system_devices_get_roles(struct ipc_connection *ipc_c,
                                  struct xrt_system_roles *out_system_roles);

xrt_result_t
ipc_call_system_devices_begin_feature(struct ipc_connection *ipc_c,
                                      enum xrt_device_feature_type type);

xrt_result_t
ipc_call_system_devices_end_feature(struct ipc_connection *ipc_c,
                                    enum xrt_device_feature_type type);

xrt_result_t
ipc_call_system_compositor_get_info(struct ipc_connection *ipc_c,
                                    struct xrt_system_compositor_info *out_info);

xrt_result_t
ipc_call_session_create(struct ipc_connection *ipc_c,
                        const struct xrt_session_info *xsi,
                        bool create_native_compositor);

xrt_result_t
ipc_call_session_poll_events(struct ipc_connection *ipc_c,
                             union xrt_session_event *out_event);

xrt_result_t
ipc_call_session_begin(struct ipc_connection *ipc_c);

xrt_result_t
ipc_call_session_end(struct ipc_connection *ipc_c);

xrt_result_t
ipc_call_session_destroy(struct ipc_connection *ipc_c);

xrt_result_t
ipc_call_space_create_semantic_ids(struct ipc_connection *ipc_c,
                                   uint32_t *out_root_id,
                                   uint32_t *out_view_id,
                                   uint32_t *out_local_id,
                                   uint32_t *out_local_floor_id,
                                   uint32_t *out_stage_id,
                                   uint32_t *out_unbounded_id);

xrt_result_t
ipc_call_space_create_offset(struct ipc_connection *ipc_c,
                             uint32_t parent_id,
                             const struct xrt_pose *offset,
                             uint32_t *out_space_id);

xrt_result_t
ipc_call_space_create_pose(struct ipc_connection *ipc_c,
                           uint32_t xdev_id,
                           enum xrt_input_name name,
                           uint32_t *out_space_id);

xrt_result_t
ipc_call_space_locate_space(struct ipc_connection *ipc_c,
                            uint32_t base_space_id,
                            const struct xrt_pose *base_offset,
                            int64_t at_timestamp,
                            uint32_t space_id,
                            const struct xrt_pose *offset,
                            struct xrt_space_relation *out_relation);

xrt_result_t
ipc_send_space_locate_spaces_locked(struct ipc_connection *ipc_c,
                                    uint32_t base_space_id,
                                    const struct xrt_pose *base_offset,
                                    uint32_t space_count,
                                    int64_t at_timestamp);

xrt_result_t
ipc_receive_space_locate_spaces_locked(struct ipc_connection *ipc_c);

xrt_result_t
ipc_call_space_locate_device(struct ipc_connection *ipc_c,
                             uint32_t base_space_id,
                             const struct xrt_pose *base_offset,
                             int64_t at_timestamp,
                             uint32_t xdev_id,
                             struct xrt_space_relation *out_relation);

xrt_result_t
ipc_call_space_destroy(struct ipc_connection *ipc_c,
                       uint32_t space_id);

xrt_result_t
ipc_call_space_mark_ref_space_in_use(struct ipc_connection *ipc_c,
                                     enum xrt_reference_space_type type);

xrt_result_t
ipc_call_space_unmark_ref_space_in_use(struct ipc_connection *ipc_c,
                                       enum xrt_reference_space_type type);

xrt_result_t
ipc_call_space_recenter_local_spaces(struct ipc_connection *ipc_c);

xrt_result_t
ipc_call_space_get_tracking_origin_offset(struct ipc_connection *ipc_c,
                                          uint32_t origin_id,
                                          struct xrt_pose *out_offset);

xrt_result_t
ipc_call_space_set_tracking_origin_offset(struct ipc_connection *ipc_c,
                                          uint32_t origin_id,
                                          const struct xrt_pose *offset);

xrt_result_t
ipc_call_space_get_reference_space_offset(struct ipc_connection *ipc_c,
                                          enum xrt_reference_space_type type,
                                          struct xrt_pose *out_offset);

xrt_result_t
ipc_call_space_set_reference_space_offset(struct ipc_connection *ipc_c,
                                          enum xrt_reference_space_type type,
                                          const struct xrt_pose *offset);

xrt_result_t
ipc_call_compositor_get_info(struct ipc_connection *ipc_c,
                             struct xrt_compositor_info *out_info);

xrt_result_t
ipc_call_compositor_predict_frame(struct ipc_connection *ipc_c,
                                  int64_t *out_frame_id,
                                  int64_t *out_wake_up_time,
                                  int64_t *out_predicted_display_time,
                                  int64_t *out_predicted_display_period);

xrt_result_t
ipc_call_compositor_wait_woke(struct ipc_connection *ipc_c,
                              int64_t frame_id);

xrt_result_t
ipc_call_compositor_begin_frame(struct ipc_connection *ipc_c,
                                int64_t frame_id);

xrt_result_t
ipc_call_compositor_discard_frame(struct ipc_connection *ipc_c,
                                  int64_t frame_id);

xrt_result_t
ipc_call_compositor_layer_sync(struct ipc_connection *ipc_c,
                               uint32_t slot_id,
                               const xrt_graphics_sync_handle_t *handles,
                               const uint32_t handle_count,
                               uint32_t *out_free_slot_id);

xrt_result_t
ipc_call_compositor_layer_sync_with_semaphore(struct ipc_connection *ipc_c,
                                              uint32_t slot_id,
                                              uint32_t semaphore_id,
                                              uint64_t semaphore_value,
                                              uint32_t *out_free_slot_id);

xrt_result_t
ipc_call_compositor_set_performance_level(struct ipc_connection *ipc_c,
                                          enum xrt_perf_domain domain,
                                          enum xrt_perf_set_level level);

xrt_result_t
ipc_call_compositor_set_thread_hint(struct ipc_connection *ipc_c,
                                    enum xrt_thread_hint hint,
                                    uint32_t thread_id);

xrt_result_t
ipc_call_compositor_get_display_refresh_rate(struct ipc_connection *ipc_c,
                                             float *out_out_display_refresh_rate_hz);

xrt_result_t
ipc_call_compositor_request_display_refresh_rate(struct ipc_connection *ipc_c,
                                                 float display_refresh_rate_hz);

xrt_result_t
ipc_call_compositor_get_reference_bounds_rect(struct ipc_connection *ipc_c,
                                              enum xrt_reference_space_type reference_space_type,
                                              struct xrt_vec2 *out_bounds);

xrt_result_t
ipc_call_swapchain_get_properties(struct ipc_connection *ipc_c,
                                  const struct xrt_swapchain_create_info *info,
                                  struct xrt_swapchain_create_properties *out_xsccp);

xrt_result_t
ipc_call_swapchain_create(struct ipc_connection *ipc_c,
                          const struct xrt_swapchain_create_info *info,
                          uint32_t *out_id,
                          uint32_t *out_image_count,
                          uint64_t *out_size,
                          bool *out_use_dedicated_allocation,
                          xrt_graphics_buffer_handle_t *handles,
                          uint32_t handle_count);

xrt_result_t
ipc_call_compositor_create_passthrough(struct ipc_connection *ipc_c,
                                       const struct xrt_passthrough_create_info *info);

xrt_result_t
ipc_call_compositor_create_passthrough_layer(struct ipc_connection *ipc_c,
                                             const struct xrt_passthrough_layer_create_info *info);

xrt_result_t
ipc_call_compositor_destroy_passthrough(struct ipc_connection *ipc_c);

xrt_result_t
ipc_call_swapchain_import(struct ipc_connection *ipc_c,
                          const struct xrt_swapchain_create_info *info,
                          const struct ipc_arg_swapchain_from_native *args,
                          const xrt_graphics_buffer_handle_t *handles,
                          const uint32_t handle_count,
                          uint32_t *out_id);

xrt_result_t
ipc_call_swapchain_wait_image(struct ipc_connection *ipc_c,
                              uint32_t id,
                              int64_t timeout_ns,
                              uint32_t index);

xrt_result_t
ipc_call_swapchain_acquire_image(struct ipc_connection *ipc_c,
                                 uint32_t id,
                                 uint32_t *out_index);

xrt_result_t
ipc_call_swapchain_release_image(struct ipc_connection *ipc_c,
                                 uint32_t id,
                                 uint32_t index);

xrt_result_t
ipc_call_swapchain_destroy(struct ipc_connection *ipc_c,
                           uint32_t id);

xrt_result_t
ipc_call_compositor_semaphore_create(struct ipc_connection *ipc_c,
                                     uint32_t *out_id,
                                     xrt_graphics_sync_handle_t *handles,
                                     uint32_t handle_count);

xrt_result_t
ipc_call_compositor_semaphore_destroy(struct ipc_connection *ipc_c,
                                      uint32_t id);

xrt_result_t
ipc_call_device_update_input(struct ipc_connection *ipc_c,
                             uint32_t id);

xrt_result_t
ipc_call_device_get_tracked_pose(struct ipc_connection *ipc_c,
                                 uint32_t id,
                                 enum xrt_input_name name,
                                 int64_t at_timestamp,
                                 struct xrt_space_relation *out_relation);

xrt_result_t
ipc_call_device_get_hand_tracking(struct ipc_connection *ipc_c,
                                  uint32_t id,
                                  enum xrt_input_name name,
                                  int64_t at_timestamp,
                                  struct xrt_hand_joint_set *out_value,
                                  int64_t *out_timestamp);

xrt_result_t
ipc_send_device_get_view_poses_locked(struct ipc_connection *ipc_c,
                                      uint32_t id,
                                      const struct xrt_vec3 *fallback_eye_relation,
                                      int64_t at_timestamp_ns,
                                      uint32_t view_count);

xrt_result_t
ipc_receive_device_get_view_poses_locked(struct ipc_connection *ipc_c,
                                         struct xrt_space_relation *out_head_relation,
                                         uint32_t *out_view_count);

xrt_result_t
ipc_call_device_get_view_poses_2(struct ipc_connection *ipc_c,
                                 uint32_t id,
                                 const struct xrt_vec3 *fallback_eye_relation,
                                 int64_t at_timestamp_ns,
                                 uint32_t view_count,
                                 struct ipc_info_get_view_poses_2 *out_info);

xrt_result_t
ipc_call_device_compute_distortion(struct ipc_connection *ipc_c,
                                   uint32_t id,
                                   uint32_t view,
                                   float u,
                                   float v,
                                   bool *out_ret,
                                   struct xrt_uv_triplet *out_triplet);

xrt_result_t
ipc_call_device_begin_plane_detection_ext(struct ipc_connection *ipc_c,
                                          uint32_t id,
                                          uint64_t plane_detection_id,
                                          uint64_t *out_out_plane_detection_id);

xrt_result_t
ipc_call_device_destroy_plane_detection_ext(struct ipc_connection *ipc_c,
                                            uint32_t id,
                                            uint64_t plane_detection_id);

xrt_result_t
ipc_call_device_get_plane_detection_state_ext(struct ipc_connection *ipc_c,
                                              uint32_t id,
                                              uint64_t plane_detection_id,
                                              enum xrt_plane_detector_state_ext *out_state);

xrt_result_t
ipc_send_device_get_plane_detections_ext_locked(struct ipc_connection *ipc_c,
                                                uint32_t id,
                                                uint64_t plane_detection_id);

xrt_result_t
ipc_receive_device_get_plane_detections_ext_locked(struct ipc_connection *ipc_c,
                                                   uint32_t *out_location_size,
                                                   uint32_t *out_polygon_size,
                                                   uint32_t *out_vertex_size);

xrt_result_t
ipc_call_device_get_presence(struct ipc_connection *ipc_c,
                             uint32_t id,
                             bool *out_presence);

xrt_result_t
ipc_call_device_set_output(struct ipc_connection *ipc_c,
                           uint32_t id,
                           enum xrt_output_name name,
                           const struct xrt_output_value *value);

xrt_result_t
ipc_send_device_set_haptic_output_locked(struct ipc_connection *ipc_c,
                                         uint32_t id,
                                         enum xrt_output_name name,
                                         const struct ipc_pcm_haptic_buffer *samples);

xrt_result_t
ipc_receive_device_set_haptic_output_locked(struct ipc_connection *ipc_c,
                                            uint32_t *out_samples_consumed);

xrt_result_t
ipc_call_device_get_output_limits(struct ipc_connection *ipc_c,
                                  uint32_t id,
                                  struct xrt_output_limits *out_limits);

xrt_result_t
ipc_send_device_get_visibility_mask_locked(struct ipc_connection *ipc_c,
                                           uint32_t id,
                                           enum xrt_visibility_mask_type type,
                                           uint32_t view_index);

xrt_result_t
ipc_receive_device_get_visibility_mask_locked(struct ipc_connection *ipc_c,
                                              uint32_t *out_mask_size);

xrt_result_t
ipc_call_device_is_form_factor_available(struct ipc_connection *ipc_c,
                                         uint32_t id,
                                         enum xrt_form_factor form_factor,
                                         bool *out_available);

xrt_result_t
ipc_call_device_get_face_tracking(struct ipc_connection *ipc_c,
                                  uint32_t id,
                                  enum xrt_input_name facial_expression_type,
                                  int64_t at_timestamp_ns,
                                  struct xrt_facial_expression_set *out_value);

xrt_result_t
ipc_call_device_get_body_skeleton(struct ipc_connection *ipc_c,
                                  uint32_t id,
                                  enum xrt_input_name body_tracking_type,
                                  struct xrt_body_skeleton *out_value);

xrt_result_t
ipc_call_device_get_body_joints(struct ipc_connection *ipc_c,
                                uint32_t id,
                                enum xrt_input_name body_tracking_type,
                                int64_t desired_timestamp_ns,
                                struct xrt_body_joint_set *out_value);

xrt_result_t
ipc_call_device_get_battery_status(struct ipc_connection *ipc_c,
                                   uint32_t id,
                                   bool *out_present,
                                   bool *out_charging,
                                   float *out_charge);

xrt_result_t
ipc_call_device_get_brightness(struct ipc_connection *ipc_c,
                               uint32_t id,
                               float *out_brightness);

xrt_result_t
ipc_call_device_set_brightness(struct ipc_connection *ipc_c,
                               uint32_t id,
                               float brightness,
                               bool relative);

#ifdef __cplusplus
}
#endif
